"use strict";

import React, { useMemo } from 'react';
import { isFunction, splitProps } from '@chakra-ui/utils';
import { useColorMode } from "../../hooks/index.js";
import { expandResponsive, isTextProp } from "../../style-system/index.js";
import { getColor, useTheme } from "../../theme/index.js";
import { ficus } from "../system/index.js";
import { getStateStyles } from "../system/get-state-styles.js";

/**
 * Determines if the given React node is a plain text or number.
 */
import { jsx as _jsx } from "react/jsx-runtime";
const isTextNode = node => typeof node === 'string' || typeof node === 'number';
function applyColorToChild(child, styles) {
  if (! /*#__PURE__*/React.isValidElement(child)) return child;

  // If it's a primitive element like <Icon />, inject color or styles
  const childProps = {
    ...(child.props || {}),
    color: child.props?.color || styles?.color
  };
  return /*#__PURE__*/React.cloneElement(child, childProps);
}

/**
 * Splits children into an array, wrapping text children in `ficus.Text` with applied styles.
 */
export function splitChildren(children, textStyles, buttonId) {
  return React.Children.map(children, (child, index) => {
    const key = `${buttonId || 'button'}-${index}`;
    if (isTextNode(child)) return /*#__PURE__*/_jsx(ficus.Text, {
      __styles: textStyles,
      children: child
    }, key);
    if (/*#__PURE__*/React.isValidElement(child)) {
      return /*#__PURE__*/_jsx(React.Fragment, {
        children: applyColorToChild(child, textStyles || {})
      }, key);
    }
    return /*#__PURE__*/_jsx(React.Fragment, {
      children: child
    }, key);
  });
}

/**
 * Custom hook for managing button behavior and styles.
 */
export function useButton(props, styles) {
  const {
    id,
    full,
    isDisabled,
    isLoading,
    children,
    loadingText,
    loaderColor
  } = props;
  const {
    theme
  } = useTheme();
  const {
    colorMode
  } = useColorMode();

  // Compute styles based on state
  const stateStyles = useMemo(() => getStateStyles({
    disabled: isDisabled || isLoading
  }, styles, colorMode), [isDisabled, isLoading, styles, colorMode]);
  const [textStyles] = splitProps(stateStyles, isTextProp);
  const responsiveFull = expandResponsive({
    full
  })(theme);

  // Memoized button styles
  const buttonStyles = useMemo(() => ({
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
    flexDirection: 'row',
    width: responsiveFull?.full ? '100%' : undefined,
    ...stateStyles
  }), [responsiveFull?.full, stateStyles]);
  const resolvedColor = getColor(textStyles?.color, theme.colors);

  // Memoized spinner styles
  const spinnerStyles = useMemo(() => ({
    position: loadingText ? 'relative' : 'absolute',
    spacing: 8,
    loaderColor: loaderColor || resolvedColor
  }), [loadingText, loaderColor]);
  return {
    buttonStyles,
    spinnerStyles,
    textStyles,
    loadingText: loadingText && /*#__PURE__*/_jsx(ficus.Text, {
      __styles: textStyles,
      children: loadingText
    }),
    /**
     * Retrieves the processed button children.
     */
    getChildren() {
      if (isFunction(children)) {
        if (isLoading) {
          /**
           * To be able to render it inside of the invisible View when loading
           */
          return children({
            pressed: false
          });
        }
        return children;
      }
      return splitChildren(children, textStyles, id);
    }
  };
}
//# sourceMappingURL=button.service.js.map