"use strict";

import { mergeWith, runIfFn, splitProps } from '@chakra-ui/utils';
import { isStyleProp } from "./system.js";
import { expandResponsive } from "./utils/index.js";
/**
 * `size` and `variant` are special props that are not resolved in the style-system.
 * We need to take care of the responsive values here.
 */
function createResolver(theme, colorMode) {
  return function resolver(config, prop, value, props) {
    const result = {};
    if (!value) {
      return;
    }
    const isMultipart = !!config.parts;
    const propValue = {
      [prop]: config[prop]?.[value]
    };
    const _styles = expandResponsive(propValue)(theme);
    const styles = runIfFn(_styles[prop], props);
    if (!styles) {
      return;
    }
    const stylesResult = colorMode && colorMode === 'dark' ? {
      ...styles,
      ...styles._dark
    } : styles;
    if (isMultipart) {
      config.parts?.forEach(part => {
        mergeWith(result, {
          [part]: stylesResult[part]
        });
      });
    }
    return stylesResult;
  };
}
function applyDarkModeStyles(style, colorMode) {
  if (typeof style !== 'object' || style === null) return style;
  let result = {};
  for (const key in style) {
    if (key === '_dark') continue;
    const value = style[key];
    if (typeof value === 'object' && value !== null) {
      result[key] = applyDarkModeStyles(value, colorMode);
    } else {
      result[key] = value;
    }
  }
  if (colorMode === 'dark' && typeof style._dark === 'object' && style._dark !== null) {
    const darkOverrides = applyDarkModeStyles(style._dark, colorMode);
    result = {
      ...result,
      ...darkOverrides
    };
  }
  return result;
}
export function resolveStyleConfig(config) {
  return props => {
    const {
      variant,
      size,
      theme,
      colorMode,
      _dark,
      ...rest
    } = props;
    const recipe = createResolver(theme, colorMode);
    const [restStyles] = splitProps(rest, isStyleProp);
    const baseStyle = runIfFn(config.baseStyle ?? {}, props);
    const baseStyleResult = applyDarkModeStyles(baseStyle, colorMode);
    return mergeWith({}, baseStyleResult, recipe(config, 'sizes', size, props), recipe(config, 'variants', variant, props), restStyles);
  };
}
//# sourceMappingURL=style-config.js.map